import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:zego_zimkit/zego_zimkit.dart';
import 'package:cached_network_image/cached_network_image.dart';

import 'chat_page_actions.dart';
import 'constants.dart';
import 'login_page.dart';
import 'main.dart';
import 'popup_home_page.dart';

class ZIMKitDemoHomePage extends StatefulWidget {
  const ZIMKitDemoHomePage({Key? key}) : super(key: key);

  @override
  State<ZIMKitDemoHomePage> createState() => _ZIMKitDemoHomePageState();
}

class _ZIMKitDemoHomePageState extends State<ZIMKitDemoHomePage> {
  var currentPageIndex = ValueNotifier<int>(0);
  var pages = <Widget>[];

  @override
  void initState() {
    super.initState();

    pages = [
      chatsPage(),
      Container(),
      Container(),
      mePage(),
    ];
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => false,
      child: Scaffold(
        appBar: AppBar(
          title: const Text('Conversations'),
          actions: const [HomePagePopupMenuButton()],
        ),
        body: ValueListenableBuilder<int>(
          valueListenable: currentPageIndex,
          builder: (context, pageIndex, _) {
            return pages[pageIndex];
          },
        ),
        bottomNavigationBar: bottomNavigationBar(),
      ),
    );
  }

  Widget bottomNavigationBar() {
    const backgroundColor = Colors.grey;
    final bottomNavItems = <BottomNavigationBarItem>[
      BottomNavigationBarItem(
        backgroundColor: backgroundColor,
        icon: Stack(
          children: [
            const Align(
              alignment: Alignment.center,
              child: Icon(Icons.chat_sharp),
            ),
            ValueListenableBuilder<int>(
              valueListenable: ZIMKit().getTotalUnreadMessageCount(),
              builder: (context, unreadMessageCount, _) {
                return unreadMessageCount > 0
                    ? Positioned(
                        top: 0,
                        right: 20,
                        child: Container(
                          decoration: const BoxDecoration(
                            shape: BoxShape.circle,
                            color: Colors.red,
                          ),
                          width: 8,
                          height: 8,
                        ),
                      )
                    : Container();
              },
            ),
          ],
        ),
        label: 'Chats',
      ),
      const BottomNavigationBarItem(
        backgroundColor: backgroundColor,
        icon: Icon(Icons.contacts),
        label: 'Contacts',
      ),
      const BottomNavigationBarItem(
        backgroundColor: backgroundColor,
        icon: Icon(Icons.compass_calibration),
        label: 'Discover',
      ),
      const BottomNavigationBarItem(
        backgroundColor: backgroundColor,
        icon: Icon(Icons.person_3),
        label: 'Me',
      ),
    ];

    return ValueListenableBuilder<int>(
      valueListenable: currentPageIndex,
      builder: (context, pageIndex, _) {
        return BottomNavigationBar(
          items: bottomNavItems,
          currentIndex: pageIndex,
          type: BottomNavigationBarType.shifting,
          onTap: (index) {
            currentPageIndex.value = index;
          },
        );
      },
    );
  }

  Widget chatsPage() {
    return ZIMKitConversationListView(
      onPressed: (context, conversation, defaultAction) {
        Navigator.push(context, MaterialPageRoute(
          builder: (context) {
            return demoMessageListPage(
              context,
              conversation,
            );
          },
        ));
      },
    );
  }

  Widget mePage() {
    return Padding(
      padding: const EdgeInsets.all(50),
      child: Column(
        children: [
          SizedBox(
            height: 50,
            child: Row(
              children: [
                SizedBox(
                  height: 40,
                  width: 40,
                  child: CachedNetworkImage(
                    imageUrl:
                        'https://robohash.org/${currentUser.id}.png?set=set4',
                    imageBuilder: (context, imageProvider) => Container(
                      decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        image: DecorationImage(
                          image: imageProvider,
                          fit: BoxFit.cover,
                        ),
                      ),
                    ),
                    progressIndicatorBuilder:
                        (context, url, downloadProgress) =>
                            CircularProgressIndicator(
                      value: downloadProgress.progress,
                    ),
                    errorWidget: (context, url, error) {
                      return const Icon(Icons.person);
                    },
                  ),
                ),
                const SizedBox(width: 10),
                Column(
                  children: [
                    Text(
                      currentUser.name,
                      textAlign: TextAlign.left,
                      style: const TextStyle(
                        fontSize: 16,
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                    const SizedBox(height: 40 - 16 - 16),
                    Text(
                      'ID:${currentUser.id}',
                      textAlign: TextAlign.left,
                      style: const TextStyle(
                        fontSize: 16,
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
          Expanded(child: Container()),
          ElevatedButton(
            onPressed: () async {
              await ZIMKit().disconnectUser();

              onUserLogout();

              if (mounted) {
                Navigator.of(context).pushReplacement(
                  MaterialPageRoute(
                    builder: (context) => const ZIMKitDemoLoginPage(),
                  ),
                );
              }
            },
            child: const Text(
              'Logout',
              style: TextStyle(
                fontSize: 16,
                fontWeight: FontWeight.bold,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

Widget demoMessageListPage(
  BuildContext context,
  ZIMKitConversation conversation,
) {
  return ZIMKitMessageListPage(
    conversationID: conversation.id,
    conversationType: conversation.type,
    onMessageSent: (ZIMKitMessage message) {
      debugPrint('onMessageSent: ${message.type.name}');
    },
    appBarActions: demoAppBarActions(context, conversation),
    onMessageItemLongPress: onMessageItemLongPress,
    messageListBackgroundBuilder: (context, defaultWidget) {
      return const ColoredBox(color: Colors.white);
    },
  );
}

Future<void> onMessageItemLongPress(
  BuildContext context,
  LongPressStartDetails details,
  ZIMKitMessage message,
  Function defaultAction,
) async {
  showCupertinoDialog(
    context: context,
    barrierDismissible: true,
    builder: (context) {
      return CupertinoAlertDialog(
        title: const Text('Confirm'),
        content: const Text('Delete or recall this message?'),
        actions: [
          CupertinoDialogAction(
            onPressed: Navigator.of(context).pop,
            child: const Text('Cancel'),
          ),
          CupertinoDialogAction(
            onPressed: () {
              ZIMKit().deleteMessage([message]);
              Navigator.pop(context);
            },
            child: const Text('Delete'),
          ),
          CupertinoDialogAction(
            onPressed: () {
              ZIMKit().recallMessage(message).catchError((error) {
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(content: Text(error.toString())),
                );
              });
              Navigator.pop(context);
            },
            child: const Text('Recall'),
          ),
        ],
      );
    },
  );
}
