import 'package:flutter/material.dart';

import 'package:zego_zim/zego_zim.dart';

import 'package:zego_zimkit/src/components/components.dart';
import 'package:zego_zimkit/src/components/defines.dart';
import 'package:zego_zimkit/src/components/messages/file_message.dart';
import 'package:zego_zimkit/src/components/messages/image_message.dart';
import 'package:zego_zimkit/src/components/messages/widgets/message_pointer.dart';
import 'package:zego_zimkit/src/services/services.dart';

class ZIMKitMessageWidget extends StatelessWidget {
  const ZIMKitMessageWidget({
    Key? key,
    this.onPressed,
    this.onLongPress,
    this.statusBuilder,
    this.avatarBuilder,
    this.timestampBuilder,
    required this.message,
  }) : super(key: key);

  final ZIMKitMessage message;

  final Widget Function(
    BuildContext context,
    ZIMKitMessage message,
    Widget defaultWidget,
  )? avatarBuilder;
  final Widget Function(
    BuildContext context,
    ZIMKitMessage message,
    Widget defaultWidget,
  )? statusBuilder;
  final Widget Function(
    BuildContext context,
    ZIMKitMessage message,
    Widget defaultWidget,
  )? timestampBuilder;
  final void Function(
    BuildContext context,
    ZIMKitMessage message,
    Function defaultAction,
  )? onPressed;
  final void Function(
    BuildContext context,
    LongPressStartDetails details,
    ZIMKitMessage message,
    Function defaultAction,
  )? onLongPress;

  // TODO default onPressed onLongPress action
  // TODO custom message
  Widget buildMessage(BuildContext context) {
    switch (message.type) {
      case ZIMMessageType.text:
        return ZIMKitTextMessage(
          onLongPress: onLongPress,
          onPressed: onPressed,
          message: message,
        );
      case ZIMMessageType.audio:
        return ZIMKitAudioMessage(
          onLongPress: onLongPress,
          onPressed: onPressed,
          message: message,
        );
      case ZIMMessageType.video:
        return ZIMKitVideoMessage(
          onLongPress: onLongPress,
          onPressed: onPressed,
          message: message,
        );
      case ZIMMessageType.file:
        return ZIMKitFileMessage(
          onLongPress: onLongPress,
          onPressed: onPressed,
          message: message,
        );
      case ZIMMessageType.image:
        return ZIMKitImageMessage(
          onLongPress: onLongPress,
          onPressed: onPressed,
          message: message,
        );
      case ZIMMessageType.revoke:
        return const Text('Recalled a message.');

      default:
        return Text(message.toString());
    }
  }

  Widget buildStatus(BuildContext context) {
    final Widget defaultStatusWidget = ZIMKitMessageStatusDot(message);
    return statusBuilder?.call(
          context,
          message,
          defaultStatusWidget,
        ) ??
        defaultStatusWidget;
  }

  Widget buildAvatar(BuildContext context) {
    final Widget defaultAvatarWidget = ClipRRect(
      borderRadius: const BorderRadius.all(Radius.circular(avatarWidth)),
      child: ZIMKitAvatar(
        userID: message.info.senderUserID,
        width: avatarWidth,
        height: avatarHeight,
      ),
    );

    return avatarBuilder?.call(
          context,
          message,
          defaultAvatarWidget,
        ) ??
        defaultAvatarWidget;
  }

  // TODO add userName or groupNickName to message
  List<Widget> localMessage(BuildContext context) {
    return [
      buildMessage(context),
      ZIMKitMessagePointer(
        messageType: message.type,
        isMine: message.isMine,
      ),
      buildStatus(context),
    ];
  }

  List<Widget> remoteMessage(BuildContext context) {
    return [
      buildAvatar(context),
      ZIMKitMessagePointer(
        messageType: message.type,
        isMine: message.isMine,
      ),
      buildMessage(context),
    ];
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(
        horizontal: itemHorizontalMargin,
        vertical: itemVerticalMargin,
      ),
      child: FractionallySizedBox(
        widthFactor: 0.66,
        alignment:
            message.isMine ? Alignment.centerRight : Alignment.centerLeft,
        child: message.isMine
            ? Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [...localMessage(context)],
              )
            : Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: [...remoteMessage(context)],
              ),
      ),
    );
  }
}
